/// <reference path="../Elements.ts" />
/// <reference path="../Classes/AnimationLayer.ts" />

// Animations must have 200x200 frames, placed in a spritesheet with a single row
// Use https://github.com/williammalone/HTML5-Photoshop-Sprite-Sheet-Creator
// To make animations of varying Frames Per Second fit, we will have breakpoints in percentage that delimitate what is to be expected.
// Example: at 25% animation length, that's the Back-Thrust moment, so all the animations should expect and behave as if that was happening right then.
// 75% would be front-thrust, etc.
// The idea is to have the base animation look "ok" without anything going on, but to also have things going on just fit mathmagically.
// Support to animations that contain the WHOLE of the events should be possible as to make it so we can make special animations for special circumstances. e.g. only encounter with the ogre
enum AnimationLayerLevel {
    BACKGROUND = 0,
    MIDDLE = 150,
    FRONT = 300
}

interface AnimationInstruction {
    name: string;
    layer?: AnimationLayerLevel | number;
}

module Elements.AnimationHandler {
    var container = document.getElementById("sceneAnimation");
    var stop = true;
    var originalSpriteSize = 200; // TODO: If ever get an actual artist, increase original sprite size to whatever new art goes for.
    export var animations : Array<AnimationLayer> = [];
    export var animationSpeed = 1500; // how long to complete in ms
    export var lastTime = 0;
    export var percentage;

    export function clearAnimations () {
        while (container.firstChild != undefined) {
            container.removeChild(container.firstChild);
        }
        animations = [];
    }

    export function addAnimations (...names : Array<AnimationInstruction>) {
        for (let i = 0; i < names.length; i++) {
            let newAnim = new AnimationLayer(names[i].name, names[i].layer);
            container.appendChild(newAnim.getElement());
            animations.push(newAnim);
        }
    }

    export function beginAnimate (targetAnimationSpeed? : number) {
        animationSpeed = targetAnimationSpeed == undefined ? 1500 : targetAnimationSpeed;
        lastTime = (new Date()).getTime();
        percentage = 0;
        fixVerticalPosition();

        container.style.display = "";
        stop = false;
        animate();
    }

    export function animate () {
        let time = (new Date()).getTime();
        let percGain = (time - lastTime) / animationSpeed;
        percentage += percGain;
        while (percentage > 1) {
            percentage -= 1;
        }
        for (let i = 0; i < animations.length; i++) {
            animations[i].updateSprite(percentage);
        }
        lastTime = time;
        if (!stop) {
            window.requestAnimationFrame(Elements.AnimationHandler.animate);
        }
    }

    function fixVerticalPosition () {
        let availHeight = getSize(true);
        //container.style.marginBottom = (availHeight - 200) + "px";
        container.style.transform = "scale(" + availHeight / originalSpriteSize + ")";
        container.style.bottom = (availHeight - originalSpriteSize) / 2 + "px";
        container.style.width = originalSpriteSize + "px";
        container.style.height = originalSpriteSize + "px";
    }

    export function getSize (fixMin = true) {
        let tab = document.getElementById("currentTurn");
        //let tab1 = document.getElementById("roomName");
        //tab1.style.marginTop = "";
        let height = window.innerHeight - tab.clientHeight;
        if (fixMin && height < originalSpriteSize) {
            //tab1.style.marginTop = (200 - height) + "px";
            //height = 200;
        }
        return height;
    }

    export function stopAnimate () {
        stop = true;
        container.style.display = "none";
        clearAnimations();
        getSize(false);
    }
    stopAnimate();
}